
#include "glframewindow.h"
#include "pic.h"			// png stuff
#include <stdlib.h>

#define MENU_HEIGHT 30
#define SLIDER_HEIGHT 30

GLFrameWindow::GLFrameWindow(int width, int height, const char *title) 
	: Fl_Window(width, height, title), 
	  menubar(0, 0, width, MENU_HEIGHT), 
	  glwindow(0, MENU_HEIGHT + SLIDER_HEIGHT, width, height),
          degreeBox(0, MENU_HEIGHT, 110, SLIDER_HEIGHT, "Degree Param:"),
          degreeCounter(110, MENU_HEIGHT, 100, SLIDER_HEIGHT, "Degree Param"),
          pointsBox(210, MENU_HEIGHT, 110, SLIDER_HEIGHT, "Points Param:"),
          pointsCounter(320, MENU_HEIGHT, 100, SLIDER_HEIGHT, "Points Param"),
          centripetalBox(420, MENU_HEIGHT, 100, SLIDER_HEIGHT, "Centripetal:"),
          centripetalCounter(520, MENU_HEIGHT, 100, SLIDER_HEIGHT, "Centripetal Param"),
          knownCurveBox(620, MENU_HEIGHT, 135, SLIDER_HEIGHT, "Known Curve Pref:"),
          knownCurveCounter(755, MENU_HEIGHT, 100, SLIDER_HEIGHT, "Known Curve Pref") {

  degreeBox.box(FL_UP_BOX);
  pointsBox.box(FL_UP_BOX);
  centripetalBox.box(FL_UP_BOX);
  knownCurveBox.box(FL_UP_BOX);

  // make the gl area resize with the frame window
  resizable(glwindow);
  end();
};

GLFrameWindow::GLWindow::GLWindow(int x, int y, int width, int height)
	: Fl_Gl_Window (x, y, width, height, "glwindow"){
}

// forward calls to GLFrameWindow
void GLFrameWindow::GLWindow::draw() {
	GLFrameWindow* frameparent = (GLFrameWindow*)parent();

	// the opengl context has been created, set rendering options
	if (!valid()) {
		frameparent->OnGLInit();
	}

	// draw stuff usign opengl
	frameparent->OnGLDraw();
}

// forward calls to GLFrameWindow
int GLFrameWindow::GLWindow::handle(int event) {
	if(Fl_Gl_Window::handle(event) == 1)
		return 1;

	GLFrameWindow* frameparent = (GLFrameWindow*)parent();
	return frameparent->handleForGLArea(event);
}

// saves a screen shot to a png file
bool GLFrameWindow::SaveToFile(const std::string& filename){
	// structure to hold window size
	typedef struct {
		int x, y, width, height;
	} Box;

	Pic* pic = NULL;
	Box wndSize;       // window size
	int currentY;      // last screen line we read from
	Pic_Pixel* pixels; // buffer to hold screen line

	// get viewport size
	glGetIntegerv(GL_VIEWPORT, (int*)&wndSize);

	// try openign the file
	pic = PicOpen(filename.c_str(), wndSize.width, wndSize.height);
	if(pic == NULL){
		printf("error opening file, %s\n",filename.c_str());
		return false;
	}

	pixels = PixelAlloc(wndSize.width); // create a buffer to hold one scan line

	// read one line at a time and save to the file
	for(currentY = wndSize.height + wndSize.y; currentY > wndSize.y; currentY--){
		glReadPixels(wndSize.x, currentY, wndSize.width, 1, GL_RGB, GL_UNSIGNED_BYTE, pixels);

		if(PicWriteLine(pic, pixels) == FALSE){
			PixelFree(pixels);
			PicClose(pic);
			printf("error writing to file\n");
			return false;
		}
	}

	PicClose(pic);
	return true;
}

