#ifndef _GLFRAMEWINDOW_H_
#define _GLFRAMEWINDOW_H_

///////////////////////////////////////////////////////////////////////////////
// a class representing both the frame window (top level, with menu, etc.)
// and the OpenGL rendering area.  This is usualy two seperate Fl classes.
// this class simplifies things
//
// shouldn't need to modify this (and it's cpp) file

#include <string>
#include <FL/gl.h>
#include <FL/Fl.H>
#include <FL/Fl_Gl_Window.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Counter.H>
#include <FL/Fl_Box.H>

class GLFrameWindow : public Fl_Window{
public:
	GLFrameWindow(int width, int height, const char *title);

	// draw stuff usign opengl
	virtual void OnGLDraw(){};
	// the opengl context has been created, set rendering options
	virtual void OnGLInit(){};
	// handle events sent to this window, or it's children
	// don't override, default should be ok
	// virtual int handle(int event){ return 0; };

	// handle events for the openGL area
	virtual int handleForGLArea(int event){ return 0; };

	Fl_Menu_Bar& getMenu(){ return menubar; };
	Fl_Gl_Window& getGL(){ return glwindow; };
        Fl_Counter &getDegreeCounter() { return degreeCounter; }
        Fl_Counter &getPointsCounter() { return pointsCounter; }
        Fl_Counter &getCentripetalCounter() { return centripetalCounter; }
        Fl_Counter &getKnownCurveCounter() { return knownCurveCounter; }

	// saves a screen shot to a png file
	bool SaveToFile(const std::string& filename);

private:
	// normaly stuff would get done in here but to make this demo simpler we'll just hide this class
	// inside our main one
	class GLWindow : public Fl_Gl_Window {
	public:
		GLWindow(int x, int y, int width, int height);

	private:
		// forwards call to GLFrameWindow::OnGLDraw and GLFrameWindow::OnGLInit
		void draw();
		// forwards calls to GLFrameWindow::handleForGLArea
		int handle(int event);
	};

	// gl drawing area
	GLWindow glwindow;
	// main menu
	Fl_Menu_Bar menubar;

        Fl_Counter degreeCounter;
        Fl_Counter pointsCounter;
        Fl_Counter centripetalCounter;
        Fl_Counter knownCurveCounter;

        Fl_Box degreeBox;
        Fl_Box pointsBox;
        Fl_Box centripetalBox;
        Fl_Box knownCurveBox;
};

#ifndef M_PI
// usualy not defined on windows
#define M_PI 3.14159265358979323846
#endif

#endif
